-- Scorchio!
-- A multi-mob fire vulnerability manager
-- Ennui@Bloodhoof-EU
-- Email: ennuilg@gmail.com
-- Based on: ScorchWatch by Liccee

local SM = AceLibrary("SharedMedia-1.0")
local RL = AceLibrary("Roster-2.1")
local PC = AceLibrary("PaintChips-2.0")
local L = AceLibrary("AceLocale-2.2"):new("Scorchio")

local SB = "Scorch_Bars"
local playerName = UnitName("player")
local versionNumber = "0.23"
local testBars = false
local mageList = { }

local options = { 
    handler = Scorchio,
    type = 'group',
    args = {
        baroptions = {
            type = 'group',
            name = L["Bar Options"],
            desc = L["Bar Options"],
            args = {
                anchor = {
                    type = 'execute',
                    name = L["Anchor"],
                    desc = L["Shows/hides the dragable anchor and test bars"],
                    func = "ToggleAnchor"
                },
                growup = {
                    type = 'toggle',
                    name = L["Grow Up"],
                    desc = L["If checked, bar will stack above the anchor, not below"],
                    get = function()
                        return Scorchio.db.profile.growUp
                    end,
                    set = "ToggleGrowUp"
                },
                scale = {
                    type = 'range',
                    name = L["Scale"],
                    desc = L["Change the scale of the bars"],
                    min = 0.5,
                    max = 2,
                    get = function()
                        return Scorchio.db.profile.scale
                    end,
                    set = function(v)
                        Scorchio.db.profile.scale = v
                        if testBars then Scorchio:ShowTestBars() end
                    end
                },
                texture = {
                    type = 'text',
                    name = L["Texture"],
                    desc = L["Change the texture of the bars"],
                    get = function() 
                        return Scorchio.db.profile.texture 
                    end,
                    set = function(v)
                        Scorchio.db.profile.texture = v
                        if testBars then Scorchio:ShowTestBars() end
                    end,
                    validate = SM:List("statusbar")
                },
                alpha = {
                    type = 'range',
                    name = L["Alpha"],
                    desc = L["Change the alpha setting of the bars"],
                    min = 0.1,
                    max = 1,
                    get = function()
                        return Scorchio.db.profile.baralpha
                    end,
                    set = function(v)
                        Scorchio.db.profile.baralpha = v
                        if testBars then Scorchio:ShowTestBars() end
                    end
                },
            },
        },
        syncreceive = {
            type = 'toggle',
            name = L["Receive Data"],
            desc = L["Receive data from other Scorchio! users"],
            get = function()
                return Scorchio.db.profile.syncreceive
            end,
            set = function()
                Scorchio.db.profile.syncreceive = not Scorchio.db.profile.syncreceive
            end
        },
        warningtime = {
            type = 'range',
            name = L["Warning Time"],
            desc = L["Length of time to be warned before Fire Vulnerability expires (Set to 0 to disable)"],
            min = 0,
            max = 30,
            step = 1,
            get = function ()
                return Scorchio.db.profile.warningtime
            end,
            set = function(v)
                Scorchio.db.profile.warningtime = v
            end
        },
        blacklist = {
            type = 'group',
            name = L["Blacklist"],
            desc = L["Blacklist mages who aren't 3/3 Imp Scorch"],
            disabled = function()
               if GetNumPartyMembers() < 1 then
                   return true
               else
                  return false
               end 
            end,
            args = {
                rebuild = {
                    type = 'execute',
                    name = L["Rebuild"],
                    desc = L["Rebuild Mage List"],
                    func = "RebuildMageList"
                },
                magelist = {
                    type = 'text',
                    name = L["Blacklist"],
                    desc = L["Blacklist mages who aren't 3/3 Imp Scorch (Uncheck mages to ignore their data)"],
                    get = function(key)
                        return Scorchio.db.profile.blackList[key]
                    end,
                    set = function(key,value)
                        Scorchio.db.profile.blackList[key] = value
                        if Scorchio.db.profile.debug.code then 
                            local bool
                            if Scorchio.db.profile.blackList[key] == false then
                                bool = "false"
                            else if Scorchio.db.profile.blackList[key] == true then
                                    bool = "true"
                                else
                                    bool = "something strange"
                                end
                            end
                            Scorchio:Print(key .. ": Set to " .. bool)
                        end
                    end,
                    multiToggle = true,
                    validate = mageList
                },
            },
        },
        debug = {
            type = 'group',
            name = "Debug",
            desc = "Debug Options",
            guiHidden = true,
            args = {
                duplicates = {
                    type = 'toggle',
                    name = "Duplicates",
                    desc = "Notify of duplicate supression.",
                    get = function()
                        return Scorchio.db.profile.debug.duplicates
                    end,
                    set = function()
                        Scorchio.db.profile.debug.duplicates = not Scorchio.db.profile.debug.duplicates
                    end
                },
                sync = {
                    type = 'toggle',
                    name = "Sync Info",
                    desc = "Display synchronisation debug information",
                    get = function()
                        return Scorchio.db.profile.debug.sync
                    end,
                    set = function()
                        Scorchio.db.profile.debug.sync = not Scorchio.db.profile.debug.sync
                    end                   
                },
                mobs = {
                    type = 'toggle',
                    name = "Mob Info",
                    desc = "Display mob debug information",
                    get = function()
                        return Scorchio.db.profile.debug.mob
                    end,
                    set = function()
                        Scorchio.db.profile.debug.mob = not Scorchio.db.profile.debug.mob
                    end
                },
                code = {
                    type = 'toggle',
                    name = "Code Info",
                    desc = "Display code debug information",
                    get = function()
                        return Scorchio.db.profile.debug.code
                    end,
                    set = function()
                        Scorchio.db.profile.debug.code = not Scorchio.db.profile.debug.code
                    end
                },
                timers = {
                    type = 'toggle',
                    name = "Timers Info",
                    desc = "Display timers debug information",
                    get = function()
                        return Scorchio.db.profile.debug.timers
                    end,
                    set = function()
                        Scorchio.db.profile.debug.timers = not Scorchio.db.profile.debug.timers
                    end
                },                  
            },
        },
    },
}

Scorchio = AceLibrary("AceAddon-2.0"):new("AceComm-2.0", "AceConsole-2.0", "AceDB-2.0", "AceEvent-2.0", "CandyBar-2.0", "FuBarPlugin-2.0", "Sink-1.0")

Scorchio:RegisterChatCommand("/scorch", options)
Scorchio.OnMenuRequest = options

Scorchio:RegisterDB("ScorchioDB", "ScorchioDBPC")
Scorchio:RegisterDefaults("profile", {
		barposition = {0, 50},
        sink10OutputSink = "ChatFrame",
        scale = 1,
        baralpha = 1,
        texture = "Blizzard",
        syncreceive = true,
        warningtime = 6,
        debug = {
            duplicates = false,
            sync = false,
            mob = false,
            code = false,
            timers = false
        },
        blackList = { }
	}
)

Scorchio:SetCommPrefix("Scorchio")

Scorchio.hasIcon = "Interface\\Icons\\Spell_Fire_SoulBurn.png"

function Scorchio:InitializeTables()
    scorchTable = { }
    casterTable = { }
    cleanTable = { }
end

function Scorchio:OnInitialize()
end

function Scorchio:OnEnable()
    self:RegisterComm(self.commPrefix, "GROUP", "ReceiveMessage")
    self:RegisterEvent("CHAT_MSG_SPELL_SELF_DAMAGE")
    self:RegisterEvent("CHAT_MSG_SPELL_PARTY_DAMAGE")
    self:RegisterEvent("CHAT_MSG_COMBAT_HOSTILE_DEATH")
    self:RegisterEvent("PLAYER_REGEN_ENABLED")
    self:RegisterEvent("PARTY_MEMBERS_CHANGED")
    self:RegisterEvent("TIMER_COMPLETE", "TimerComplete")
    self.anchor = self:CreateAnchor()
    self:RegisterCandyBarGroup(SB)
    self:SetCandyBarGroupPoint(SB, "TOP", self.anchor, "BOTTOM", -7, 0)
    self:SetCandyBarGroupGrowth(SB, false)
    self:InitializeTables()
    if self.db.profile.blackList[playerName] == nil then self.db.profile.blackList[playerName] = true end
end

function Scorchio:OnDisable()
end

function Scorchio:ShowTestBars()
    for c = 1,2 do
        local name = "Test" .. c
        self:RegisterCandyBar(name, 30 / c, name)
        self:RegisterCandyBarWithGroup(name, SB)
        if self.db.profile.debug.code then self:Print("PC: ", PC) end
        self:SetCandyBarColor(name, "green", self.db.profile.baralpha)
        self:SetCandyBarBackgroundColor(name, "red", self.db.profile.baralpha, 0)
        self:SetCandyBarFade(name, -1)
        self:SetCandyBarScale(name, self.db.profile.scale)
        self:SetCandyBarTexture(name, SM:Fetch("statusbar", self.db.profile.texture))
        self:StartCandyBar(name)
        testBars = true
    end
end

function Scorchio:HideTestBars()
       for c = 1,2 do
            local name = "Test" .. c
            self:UnregisterCandyBarWithGroup(name,SB)
            self:UnregisterCandyBar(name)
        end
        testBars = false
    end

function Scorchio:ToggleAnchor()
	if self.anchor:IsVisible() then
		self.anchor:Hide()
        self:HideTestBars()
	else
		self.anchor:Show()
        self:ShowTestBars()
	end
end

function Scorchio:CreateAnchor()
	local f = CreateFrame("Button", nil, UIParent)
		f:SetWidth(160)
		f:SetHeight(25)
		
		f.owner = self
		
		if self.db.profile.barposition.x and self.db.profile.barposition.y then
		    f:ClearAllPoints()
			f:SetPoint("TOPLEFT", UIParent, "TOPLEFT", self.db.profile.barposition.x, self.db.profile.barposition.y)
		else
            f:SetPoint("CENTER", UIParent, "CENTER", 0, 50)
		end
	
		f:SetScript("OnDragStart", function() this:StartMoving() end )
		f:SetScript("OnDragStop",
			function()
				this:StopMovingOrSizing()
				local _, _, _, x, y = this:GetPoint()
				this.owner.db.profile.barposition.x = math.floor(x)
				this.owner.db.profile.barposition.y = math.floor(y)
	        end)
	
		f:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background",
	                                            edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
	                                            tile = false, tileSize = 16, edgeSize = 16,
	                                            insets = { left = 5, right =5, top = 5, bottom = 5 }})
		f:SetBackdropColor(1,0,0,.6)
		f:SetMovable(true)
		f:RegisterForDrag("LeftButton")
	
		f.Text = f:CreateFontString(nil, "OVERLAY")
		f.Text:SetFontObject(GameFontNormalSmall)
		f.Text:ClearAllPoints()
		f.Text:SetTextColor(1, 1, 1, 1)
		f.Text:SetWidth(160)
		f.Text:SetHeight(25)
		f.Text:SetPoint("TOPLEFT", f, "TOPLEFT")
		f.Text:SetJustifyH("CENTER")
		f.Text:SetJustifyV("MIDDLE")
		f.Text:SetText("Scorchio! v" .. versionNumber)
		f:Hide()
	return f
end

function Scorchio:ToggleGrowUp()
	self.db.profile.growUp = not self.db.profile.growUp
	if self.db.profile.growUp then
		self:SetCandyBarGroupPoint(SB, "BOTTOM", self.anchor, "TOP", -7, 0)
		self:SetCandyBarGroupGrowth(SB, true)
        if testBars then self:ShowTestBars() end
	else
		self:SetCandyBarGroupPoint(SB, "TOP", self.anchor, "BOTTOM", -7, 0)
		self:SetCandyBarGroupGrowth(SB, false)
        if testBars then self:ShowTestBars() end
	end
end

function Scorchio:CHAT_MSG_SPELL_SELF_DAMAGE(msg)
	-- If the Scorch spell hits / crits
	local spell1 = string.match(arg1,'Your (.+) crits')
	local spell2 = string.match(arg1,'Your (.+) hits')
    local mob = string.match(arg1,'^.+its (.+) for')
    castTime = GetTime()
    if (spell1 == "Scorch" or spell2 == "Scorch") then
        if self.db.profile.debug.code then self:Print(playerName, casterTable[playerName], castTime, "SELF") end
        if casterTable[playerName] == nil or casterTable[playerName] < castTime - 1.2 then
            if self.db.profile.blackList[playerName] then
                if self.db.profile.debug.sync then self:Print("Sending: " .. playerName .. ", " .. mob) end
                self:SendCommMessage("GROUP", playerName, mob)
                casterTable[playerName] = castTime
                self:UpdateTable(mob, castTime)
                self:CreateBar(scorchTable[mob])
            else
                if self.db.profile.debug.sync then self:Print(playerName .. " blacklisted. Ignoring event.") end
            end
        else
            if self.db.profile.debug.duplicates then self:Print("Duplicate event ignored (" .. playerName ..") SELF") end
        end 
    end
end

function Scorchio:CHAT_MSG_SPELL_PARTY_DAMAGE(msg)
	local spell1 = string.match(arg1,"^.+ 's (.+) crits")
	local spell2 = string.match(arg1,"^.+ 's (.+) hits")
    local mob = string.match(arg1,'^.+its (.+) for')
    local caster = string.match(arg1,"^(.+) 's")
    local castTime = GetTime()
    if (spell1 == "Scorch" or spell2 == "Scorch") then
        if self.db.profile.debug.code then self:Print(caster, casterTable[caster], castTime, "PARTY") end
        if casterTable[caster] == nil or casterTable[caster] < castTime - 1.2 then
            if self.db.profile.blackList[caster] then 
                if self.db.profile.debug.sync then self:Print("Sending: " .. caster .. ", " .. mob) end
                self:SendCommMessage("GROUP", caster, mob) 
                casterTable[caster] = castTime
                self:UpdateTable(mob, castTime)
                self:CreateBar(scorchTable[mob])
            else
                if self.db.profile.debug.sync then self:Print(caster .. " blacklisted. Ignoring event.") end
            end
        else
            if self.db.profile.debug.duplicates then self:Print("Duplicate event ignored (" .. caster ..") PARTY") end
        end                
    end
end

function Scorchio:ReceiveMessage(prefix, sender, distribution, arg1, arg2)
    if not self.db.profile.syncreceive then return end    
    local castTime = GetTime()
    local caster = arg1
    local mob = arg2
    if self.db.profile.debug.sync then self:Print("Receiving: " .. caster .. ", " .. mob) end
    if self.db.profile.debug.code then self:Print(caster, casterTable[caster], castTime, "SYNC") end
    if casterTable[caster] == nil or casterTable[caster] < castTime - 1.2 then
        if self.db.profile.blackList[caster] then
            casterTable[caster] = castTime
            self:UpdateTable(mob, GetTime())
            self:CreateBar(scorchTable[mob])
        else
            if self.db.profile.debug.sync then self:Print(caster .. " blacklisted. Ignoring event.") end
        end
    else
        if self.db.profile.debug.duplicates then self:Print("Duplicate event ignored (" .. caster ..") SYNC") end
    end                
end

function Scorchio:CHAT_MSG_COMBAT_HOSTILE_DEATH(msg)
	local mob = string.match(arg1,'(.+) is slain by')
	if (mob == nil) then
        mob = string.match(arg1,'You have slain (.+)!')
  	end
	if (mob == nil) then
	    mob = string.match(arg1,'(.+) is destroyed.')
	end
	if (mob == nil) then
		mob = string.match(arg1,'(.+) dies.')
	end
    if mob ~= nil and scorchTable[mob] ~= nil then
        if self.db.profile.debug.mob then self:Print(mob .. " dies.") end
        scorchTable[mob] = nil
        self:UnregisterCandyBarWithGroup(mob,SB)
        self:UnregisterCandyBar(mob)
    end
end

function Scorchio:PLAYER_REGEN_ENABLED()
    for k,v in pairs(scorchTable) do
        self:UnregisterCandyBarWithGroup(k,SB)
        self:UnregisterCandyBar(k)
    end
    for k,v in pairs(cleanTable) do
        self:UnregisterCandyBarWithGroup(k,SB)
        self:UnregisterCandyBar(k)
    end
    self:InitializeTables()
end

function Scorchio:PARTY_MEMBERS_CHANGED()
    Scorchio:RebuildMageList()
end

function Scorchio:RebuildMageList()
    local tmp = { }
    local count = 0
    for unit in RL:IterateRoster(false) do
        if self.db.profile.debug.code then self:Print(unit.name, unit.class) end
        if unit.class == "MAGE" then
            table.insert(tmp, unit.name)
            count = count + 1
        end
    end
    mageList = tmp
    for k,v in pairs(mageList) do
        if self.db.profile.blackList[v] == nil then
            self.db.profile.blackList[v] = true
            if self.db.profile.debug.code then self:Print("New mage: " .. v) end
        end
    end
    if self.db.profile.debug.code then self:Print(count .. " mages added.") end
    options.args.blacklist.args.magelist.validate = mageList
end
    
function Scorchio:UpdateTable(mob, time)
    if scorchTable[mob] == nil then
        if self.db.profile.debug.mob then self:Print("New mob: " .. mob) end
        scorchTable[mob] = { mob, GetTime(), 1 }
    else
        local scorchStack = scorchTable[mob][3]
        if scorchStack < 5 then
            scorchStack = scorchStack + 1
        end
        scorchTable[mob] = { mob, GetTime(), scorchStack }
    end
    if self.db.profile.warningtime > 0 then Scorchio:ScheduleEvent("TIMER_COMPLETE",(30 - self.db.profile.warningtime), mob) end
end

function Scorchio:TimerComplete(mob)
    local timertime = GetTime()
    if scorchTable[mob] then
        if self.db.profile.debug.timers then self:Print(mob,scorchTable[mob][2], timertime, scorchTable[mob][2] - timertime) end
        if scorchTable[mob][2] - timertime < (-30 + self.db.profile.warningtime) then
            self:Pour(L["Rescorch "] .. mob .. "!", 1, 0.647, 0)
        end
    end
end

function Scorchio:CreateBar(scorchTarget)
    local mob = scorchTarget[1]
    local elapsedTime = GetTime() - scorchTarget[2]
	local remainingTime = math.floor (30 - elapsedTime)
    local scorchStack = scorchTarget[3]
    local text = mob .. "  (x" .. scorchStack .. ")"
    local name = mob
    local ExpFun = function (mob)
        self:Pour(L["Fire Vulnerability Expired on "] .. mob, 1, 0, 0)
        cleanTable[mob] = true
        scorchTable[mob] = nil
    end
    self:RegisterCandyBar(name, remainingTime, text)
    self:RegisterCandyBarWithGroup(name, SB)
    self:SetCandyBarColor(name, "green", self.db.profile.baralpha)
    self:SetCandyBarBackgroundColor(name, "red", self.db.profile.baralpha)
    self:SetCandyBarFade(name, -1)
    self:SetCandyBarScale(name, self.db.profile.scale)
    self:SetCandyBarTexture(name, SM:Fetch("statusbar", self.db.profile.texture))
    self:SetCandyBarCompletion(name, ExpFun, mob)
    self:StartCandyBar(name)
end


function Scorchio:Debug()
    for k,v in pairs(scorchTable) do self:Print(v[1],v[2],v[3]) end
end